@@ -114,6 +114,8 @@ static void matchline(char* line, struct entry* e)
 			e->match_ttl = true;
 		} else if(str_keyword(&parse, "DO")) {
 			e->match_do = true;
+		} else if(str_keyword(&parse, "CO")) {
+			e->match_co = true;
 		} else if(str_keyword(&parse, "noedns")) {
 			e->match_noedns = true;
 		} else if(str_keyword(&parse, "ednsdata")) {
@@ -202,6 +204,9 @@ static void replyline(char* line, ldns_pkt *reply)
 		} else if(str_keyword(&parse, "DO")) {
 			ldns_pkt_set_edns_udp_size(reply, 4096);
 			ldns_pkt_set_edns_do(reply, true);
+		} else if(str_keyword(&parse, "CO")) {
+			ldns_pkt_set_edns_udp_size(reply, 4096);
+			ldns_pkt_set_edns_co(reply, true);
 		} else {
 			error("could not parse REPLY: '%s'", parse);
 		}
@@ -246,6 +251,7 @@ static struct entry* new_entry(void)
 	e->match_all = false;
 	e->match_ttl = false;
 	e->match_do = false;
+	e->match_co = false;
 	e->match_noedns = false;
 	e->match_serial = false;
 	e->ixfr_soa_serial = 0;
@@ -794,6 +800,10 @@ find_match(struct entry* entries, ldns_pkt* query_pkt,
 			verbose(3, "no DO bit set\n");
 			continue;
 		}
+		if(p->match_co && !ldns_pkt_edns_co(query_pkt)) {
+			verbose(3, "no CO bit set\n");
+			continue;
+		}
 		if(p->match_noedns && ldns_pkt_edns(query_pkt)) {
 			verbose(3, "bad; EDNS OPT present\n");
 			continue;
